;------------------------------------------------------------------------------
; Decoding RC5 remote control infrared signal using AVR microcontroller
; http://avr-mcu.dxp.pl
; (c) Radoslaw Kwiecien
;------------------------------------------------------------------------------

#define CYCLESPERUS	(F_CPU/1000000)
#define PRESCALE	8
#define TICKSPERUS(x) ((x*CYCLESPERUS)/PRESCALE)
#define TIMERVALUE(x) (65535-(TICKSPERUS(x)))


#define RC5PINREG	PIND
#define RC5PIN		2

#define RC5Ready
#define RC5Toggle
;------------------------------------------------------------------------------
; Data segment
;------------------------------------------------------------------------------
.dseg
RC5Command:  .db 1
RC5Address:  .db 1
RC5Status:	 .db 1

RC5Command1: .db 1
RC5Address1: .db 1
RC5Counter:  .db 1

;------------------------------------------------------------------------------
; Code segment
;------------------------------------------------------------------------------
.cseg
RC5Init:
	ldi		r16,0x02
	out		MCUCR,r16

	ldi		r16,0x40
	out		GIMSK,r16

    clr     r16
    sts     RC5Status, r16
   	sei

    ret

;------------------------------------------------------------------------------
;
;------------------------------------------------------------------------------
WaitForRC5:
    push    r16
WaitForRC5Loop:    
    lds     r16,RC5Status
    sbrs    r16,0
    rjmp    WaitForRC5Loop
    andi    r16,0xFE
    sts     RC5Status,r16
    pop     r16
    ret
;------------------------------------------------------------------------------
; External interrupt from first falling edge RC5 signal
;------------------------------------------------------------------------------
ExtInt0:
    push    r16                         ; Save used register on stack

    clr		r16                         ;   
    sts		RC5Address,  r16            ;
    sts		RC5Command,  r16            ;
    sts		RC5Address1, r16            ; Clear all used variables
    sts		RC5Command1, r16            ;
    sts		RC5Counter,  r16            ;
    sts     RC5Status, r16              ;

    in      r16,TIMSK                   ;
	ori		r16,0x80                    ; Enable interrupt from Timer overflow
	out		TIMSK, r16                  ;

    ldi		r16,HIGH(TIMERVALUE(443))   ; First timer overflow after 444us
    out		TCNT1H,r16                  ; (quarter bit time)
    ldi		r16,LOW(TIMERVALUE(443))    ;
    out		TCNT1L,r16			        ;
    ldi		r16,(1 << CS11)             ; Start timer (prescale 8)
    out		TCCR1B, r16                 ;
  
    pop     r16                         ; restore register from stack
    reti

;------------------------------------------------------------------------------
; Interrupt from Timer 1 overflow
;------------------------------------------------------------------------------
Tim1Overflow:
    push    r16                         ;
    push    r17                         ; Save used registers on stack    

	ldi		r16,HIGH(TIMERVALUE(883))	; Overflow after 886 microseconds
	out		TCNT1H,r16					; (half bit time)
	ldi		r16,LOW(TIMERVALUE(883))	;
	out		TCNT1L,r16					;

    in      r16,GIMSK
	andi    r16,0b10111111              ; Disable INT0
	out		GIMSK,r16					;

	lds		r16,RC5Counter				;
	inc		r16							; Incremet counter
	sts		RC5Counter,r16				;

	cpi		r16,1						;
	brne	RC5IsEndFrame				; Exit if first tick
	rjmp    RC5Exit

RC5IsEndFrame:
	cpi		r16,28						; If counter == 28, go to EndFrame
	breq	RC5EndFrame					;
RC5IsCommand:
	cpi		r16,16						; If counter >= 16 go to GetCommand
	brsh	RC5GetCommand				;
;----------------------------
; Receive RC5 Address
;---------------------------- 
RC5GetAddress:
	sbrc	r16,0						; skip if counter value is even
	rjmp	RC5GetAddressN					; jump to GetAddressN

	clc                                 ; 0
	sbic	RC5PINREG,RC5PIN            ; skip if input is clear
	sec                                 ; 1
	lds		r16, RC5Address
	rol		r16
	sts		RC5Address, r16
	rjmp    RC5Exit

RC5GetAddressN:
	clc									; 0
	sbis	RC5PINREG,RC5PIN			; skip if input is clear
	sec									; 1
	lds		r16, RC5Address1
	rol		r16
	sts		RC5Address1, r16			; rotate received bit into RC5Cmd
	rjmp    RC5Exit
;----------------------------
; Receive RC5 Command
;----------------------------
RC5GetCommand:
	sbrc	r16,0						; skip if counter value is even
	rjmp	RC5GetCommandN					; jump to GetCommandN

	clc									; 0
	sbic	RC5PINREG,RC5PIN			; skip if input is clear
	sec									; 1
	lds		r16, RC5Command
	rol		r16
	sts		RC5Command, r16				; rotate received bit into RC5Cmd
	rjmp    RC5Exit

RC5GetCommandN:
	clc
	sbis	RC5PINREG,RC5PIN
	sec
	lds		r16, RC5Command1
	rol		r16
	sts		RC5Command1, r16			; rotate received bit into RC5Cmd
	rjmp    RC5Exit
;----------------------------
; End of RC5 frame
;----------------------------
RC5EndFrame:
	ldi		r16,0						; 
	out		TCCR1B, r16					; Stop timer

	ldi		r16,0						;
	sts		RC5Counter,r16				; Clear Counter

    in      r16,EIFR
    ori     r16,0x40
	out		EIFR,r16					; Clear INT0 flag

    in      r16,GIMSK                   ;
	ori		r16,0x40					; Enable INT0
	out		GIMSK,r16					;
	
	lds		r16,RC5Command
	lds		r17,RC5Command1
	cp		r16, r17					; Compare 
	brne	RC5Error					; Exit if different (not RC5 code)

	andi	r16,0x3F					; Mask not important bits
	sts		RC5Command, r16				; and store command

	lds		r16,RC5Address
	lds		r17,RC5Address1
	cp		r16,r17				    	; Compare
	brne	RC5Error					; Exit if different (not RC5 code)

    sbrs    r16,5                       ; check Toggle bit
    rjmp    RC5MaskAddress              ; if zrero jump to MaskAddres
    lds     r17,RC5Status               ; 
    ori     r17,0x80                    ; otherwise set T flag in status
    sts     RC5Status,r17               ;

RC5MaskAddress:
	andi	r16,0x1F					; Mask not important bits
	sts		RC5Address,r16				; and store address
    
    lds     r16,RC5Status
    ori		r16,1					    ; Set status bit
    sts     RC5Status,r16
RC5Exit:
    pop     r17                         ; Restore registers from stack
    pop     r16                         ;
	reti
	
RC5Error:
    lds     r16, RC5Status              ;
    andi    r16,0xFE                    ; Clear READY flag in status byte
    sts     RC5Status,r16               ;
	rjmp    RC5Exit
;------------------------------------------------------------------------------
; End of RC5.asm file
;------------------------------------------------------------------------------
